/*
 *   Copyright (c) 2000 S. Noda (HZG02104@nifty.ne.jp)
 *
 *   Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose is hereby granted provided
 * that the above copyright notice and this permission notice appear
 * in all copies of the software and related documentation.
 *
 *                          NO WARRANTY
 *
 *   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES;
 * WITHOUT EVEN THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

#ifndef __ZIPDLL_H__
#define __ZIPDLL_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned char uch;		/* unsigned 8-bit value */
typedef unsigned long ulg;		/* unsigned 32-bit value */
#define PATH_MAX 128

#define BUFFSIZE (16 * 1024)	/* 16kB */

typedef struct _zip_ver {
	uch major;				/* e.g., integer 5 */
	uch minor;				/* e.g., 2 */
	uch patchlevel; 		/* e.g., 0 */
	uch not_used;
} _zip_version_type;

typedef struct _ZpVer {
	ulg structlen;			/* length of the struct being passed */
	ulg flag;				/* bit 0: is_beta	bit 1: uses_zlib */
	char betalevel[10]; 	/* e.g., "g BETA" or "" */
	char date[20];			/* e.g., "4 Sep 95" (beta) or "4 September 1995" */
	char zlib_version[10];	/* e.g., "0.95" or NULL */
	_zip_version_type zip;
	_zip_version_type os2dll;
	_zip_version_type windll;
} ZpVer;

typedef int (WINAPI DLLPRNT) (LPSTR, unsigned long);
typedef int (WINAPI DLLPASSWORD) (LPSTR, int, LPCSTR, LPCSTR);
typedef int (WINAPI DLLSERVICE) (LPCSTR, unsigned long);
typedef int (WINAPI DLLCOMMENT)(LPSTR);

typedef struct {		/* zip options */
	LPSTR Date; 			/* Date to include after */
	LPSTR szRootDir;		/* Directory to use as base for zipping */
	LPSTR szTempDir;		/* Temporary directory used during zipping */
	BOOL fTemp; 			/* Use temporary directory '-b' during zipping */
	BOOL fSuffix;			/* include suffixes (not implemented) */
	BOOL fEncrypt;			/* encrypt files */
	BOOL fSystem;			/* include system and hidden files */
	BOOL fVolume;			/* Include volume label */
	BOOL fExtra;			/* Exclude extra attributes */
	BOOL fNoDirEntries; 	/* Do not add directory entries */
	BOOL fExcludeDate;		/* Exclude files newer than specified date */
	BOOL fIncludeDate;		/* Include only files newer than specified date */
	BOOL fVerbose;			/* Mention oddities in zip file structure */
	BOOL fQuiet;			/* Quiet operation */
	BOOL fCRLF_LF;			/* Translate CR/LF to LF */
	BOOL fLF_CRLF;			/* Translate LF to CR/LF */
	BOOL fJunkDir;			/* Junk directory names */
	BOOL fGrow; 			/* Allow appending to a zip file */
	BOOL fForce;			/* Make entries using DOS names (k for Katz) */
	BOOL fMove; 			/* Delete files added or updated in zip file */
	BOOL fDeleteEntries;	/* Delete files from zip file */
	BOOL fUpdate;			/* Update zip file--overwrite only if newer */
	BOOL fFreshen;			/* Freshen zip file--overwrite only */
	BOOL fJunkSFX;			/* Junk SFX prefix */
	BOOL fLatestTime;		/* Set zip file time to time of latest file in it */
	BOOL fComment;			/* Put comment in zip file */
	BOOL fOffsets;			/* Update archive offsets for SFX files */
	BOOL fPrivilege;		/* Use privileges (WIN32 only) */
	BOOL fEncryption;		/* TRUE if encryption supported, else FALSE. this is a read only flag */
	int  fRecurse;			/* Recurse into subdirectories. 1 => -r, 2 => -R */
	int  fRepair;			/* Repair archive. 1 => -F, 2 => -FF */
	char fLevel;			/* Compression level (0 - 9) */
} ZPOPT, far *LPZPOPT;

typedef struct {
	int  argc;				/* Count of files to zip */
	LPSTR lpszZipFN;		/* name of archive to create/update */
	char **FNV; 			/* array of file names to zip up */
} ZCL, far *LPZCL;

typedef struct {
	DLLPRNT *print;
	DLLCOMMENT *comment;
	DLLPASSWORD *password;
	DLLSERVICE *ServiceApplication;
} ZIPUSERFUNCTIONS, far * LPZIPUSERFUNCTIONS;

typedef void  (WINAPI *ZP_VERSION)(ZpVer far *);		/* ZpVersion */
typedef int   (WINAPI *ZP_INIT)(LPZIPUSERFUNCTIONS);	/* ZpInit */
typedef BOOL  (WINAPI *ZP_SETOPTIONS)(LPZPOPT); 		/* ZpSetOptions */
typedef ZPOPT (WINAPI *ZP_GETOPTIONS)(void);			/* ZpGetOptions */
typedef int   (WINAPI *ZP_ARCHIVE)(ZCL);				/* ZpArchive */

#ifdef __cplusplus
}
#endif

BOOL LoadZipLibrary();
BOOL CheckZipLibrary();
int InitZipLibrary();
void FreeZipLibrary();

void LoadZipOptions();
void ChangeZipOptions();
void SaveZipOptions();

void CreateStatusDlg();
void DestroyStatusDlg();
int WINAPI UpdateStatusDlg(char far *buf, unsigned long size);

int WINAPI PasswordPrompt(LPSTR p, int n, LPCSTR m, LPCSTR name);
int WINAPI CommentPrompt(char far *szBuf);

char *GetFileName(char *Path);
void RemoveFileName(char *Path, char *Buf);

int DoArc(CString stFileName, HDROP hDropInfo);

#endif /* __ZIPDLL_H__ */
